package rt.lanyo.game.ddz;

import java.util.HashMap;

import rt.lanyo.game.ddz.mgr.ImgMgr;
import rt.lanyo.game.ddz.mgr.OnlineMgr;
import rt.lanyo.game.ddz.mgr.SaveMgr;
import rt.lanyo.game.ddz.mgr.OnlineMgr.Callback;
import rt.lanyo.game.ddz.model.Card;
import rt.lanyo.game.ddz.service.MusicService;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.vo.CmdVO;
import rt.lanyo.game.ddz.widget.MyDialog;
import android.app.Activity;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.graphics.Typeface;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.SeekBar;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.SeekBar.OnSeekBarChangeListener;
/**
 * Activity的父类,主要负责处理一些公共设置
 * @author MJ
 */
public abstract class Base extends Activity implements View.OnClickListener,OnCheckedChangeListener, 
OnSeekBarChangeListener, OnDismissListener, Callback{
	//交给子类去实现
	public void onStartTrackingTouch(SeekBar seekBar) {}
	public void onStopTrackingTouch(SeekBar seekBar) {}
	public static final byte BRIGHTNESS_BAR_MAX = 100;
	private float curBrightness;
	public static final byte DIALOG_EXIT = -1;
	public static final byte DIALOG_NET_ERROR = -2;
	public static final byte TIME_OUT = -3;
	
	private Dialog settingDialog;
	private SeekBar volumnBar;
	private SeekBar brightnessBar;
	private CheckBox musicBox;
	private CheckBox soundBox;
	private CheckBox vibrateBox;
	private CheckBox animBox;
	private SaveMgr saveMgr;
	private MyDialog forceExitDialog;
	private MyDialog timeOutDialog;
	protected abstract void create();
	
	public Handler handler = new Handler() {
		public void handleMessage(Message msg) {
			handle(msg);
		}
	};
	
	public void execute(final boolean success, final CmdVO c) {
		post(new Runnable() {
			public void run() {
				deal(success, c);
			}
		});
	}
	public void post(Runnable r){
		handler.post(r);
	}
	public void sendMsg(Message msg){
		handler.sendMessage(msg);
	}
	public void sendMsg(int what){
		handler.sendEmptyMessage(what);
	}
	protected void handle(Message msg) {
		if(msg.what==TIME_OUT) {
			if(this.getClass()==Game.class || this.getClass()==Room.class)
				timeOutDialog = MyDialog.create(R.string.tip, R.string.time_out, R.string.sure);
			
		}
	}
	protected void deal(boolean success, CmdVO c) {
		if(CmdVO.FORCE_EXIT.equals(c.cmd)) { //被强制退出
			if(this.getClass()==Game.class || this.getClass()==Room.class)
			forceExitDialog = MyDialog.create(R.string.tip, R.string.force_exit, R.string.sure);
		} 
	}
	
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setVolumeControlStream(AudioManager.STREAM_MUSIC); 
        
        if(G.activityMap==null) G.activityMap = new HashMap<Object, Activity>();
        G.context = this;
        G.activityMap.put(this.getClass(), this);
        
        create();//由子类去实现
	}

	protected void onResume() {
		super.onResume();
		SaveMgr mgr = new SaveMgr(this);
        setBrightness(mgr.get(SaveMgr.BRIGHTNESS, G.INIT_BRIGHTNESS));  //设置屏幕亮度
		G.context = this;
		if(G.normalType==null) G.normalType = Typeface.SANS_SERIF;
		if(G.numType==null) G.numType = Typeface.createFromAsset(getAssets(), "fonts/num.ttf");
		
	}
	
	public void openSettingDialog() {
		saveMgr = new SaveMgr(this);
		
		LayoutInflater mLayoutInflater = (LayoutInflater) getSystemService(LAYOUT_INFLATER_SERVICE);
		ViewGroup menuView = (ViewGroup) mLayoutInflater.inflate(R.layout.dialog_setting, null, true);
		settingDialog = new Dialog(this, R.style.Theme_Dialog);
		settingDialog.setOnDismissListener(this);
		settingDialog.show();
		settingDialog.getWindow().setContentView(menuView);
		
		volumnBar = (SeekBar) settingDialog.findViewById(R.id.volumn_bar);
		volumnBar.setMax(MusicService.getAudioManager().getStreamMaxVolume(AudioManager.STREAM_MUSIC));
		volumnBar.setProgress(MusicService.getAudioManager().getStreamVolume(AudioManager.STREAM_MUSIC));
		volumnBar.setOnSeekBarChangeListener(this);
		
		SaveMgr mgr = new SaveMgr(this);
		curBrightness = mgr.get(SaveMgr.BRIGHTNESS, G.INIT_BRIGHTNESS);
		brightnessBar = (SeekBar) settingDialog.findViewById(R.id.brightness_bar);
		brightnessBar.setMax(BRIGHTNESS_BAR_MAX);
		brightnessBar.setProgress((int)(curBrightness*BRIGHTNESS_BAR_MAX));
		brightnessBar.setOnSeekBarChangeListener(this);
		
		musicBox =  (CheckBox) settingDialog.findViewById(R.id.music_box);
		musicBox.setChecked(mgr.get(SaveMgr.MUSIC, true));
		musicBox.setOnCheckedChangeListener(this);
		
		soundBox =  (CheckBox) settingDialog.findViewById(R.id.sound_box);
		soundBox.setChecked(mgr.get(SaveMgr.SOUND, true));
		soundBox.setOnCheckedChangeListener(this);
		
		vibrateBox =  (CheckBox) settingDialog.findViewById(R.id.vibrate_box);
		vibrateBox.setChecked(mgr.get(SaveMgr.VIBRATE, true));
		vibrateBox.setOnCheckedChangeListener(this);
		
		animBox =  (CheckBox) settingDialog.findViewById(R.id.anim_box);
		animBox.setChecked(mgr.get(SaveMgr.ANIM, true));
		animBox.setOnCheckedChangeListener(this);
	}
	
	public void onClick(View v) {
		MusicService.playSound(Card.CLICK);
		if(v==MyDialog.getBtn(forceExitDialog)) {
			MyDialog.dismiss(forceExitDialog);
		} else if(v==MyDialog.getBtn(timeOutDialog)) {
			MyDialog.dismiss(timeOutDialog);
		}
	}

	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
		if(buttonView == musicBox){
			MusicService.setVoiceMode(isChecked, MusicService.needSound, MusicService.needVibrate);
			saveMgr.update(SaveMgr.MUSIC, isChecked);
		} else if(buttonView == soundBox){
			MusicService.setVoiceMode(MusicService.needMusic, isChecked, MusicService.needVibrate);
			saveMgr.update(SaveMgr.SOUND, isChecked);
		} else if(buttonView == vibrateBox){
			MusicService.setVoiceMode(MusicService.needMusic, MusicService.needSound, isChecked);
			saveMgr.update(SaveMgr.VIBRATE, isChecked);
		} else if(buttonView == animBox){
			ImgMgr.needAnim = isChecked;
			saveMgr.update(SaveMgr.ANIM, isChecked);
		}
		
		MusicService.playSound(Card.CLICK);
	}

	public void onProgressChanged(SeekBar seekBar, int progress,
			boolean fromUser) {
		if(fromUser) {
			if(volumnBar == seekBar){
				MusicService.setVolumn(progress);
			} else if(brightnessBar == seekBar){
				curBrightness = Integer.valueOf(progress).floatValue()/Integer.valueOf(BRIGHTNESS_BAR_MAX).floatValue();
				setBrightness(curBrightness);
			}
		}
	}
	
	public void onDismiss(DialogInterface dialog) {
		if(dialog==settingDialog) {
			saveMgr.update(SaveMgr.BRIGHTNESS, curBrightness);
			saveMgr.commit();
		} else if(dialog==MyDialog.getDialog(forceExitDialog) || dialog==MyDialog.getDialog(timeOutDialog)) {
			Game.exitGame = true;
			if(dialog==MyDialog.getDialog(timeOutDialog)) OnlineMgr.close();
			Activity game = G.activityMap.get(Game.class);
			Activity room = G.activityMap.get(Room.class);
			if(game!=null) game.finish();
			if(room!=null) room.finish();
		}
	}
	
	private void setBrightness(float f) {
		if(f<G.MIN_BRIGHTNESS) f = G.MIN_BRIGHTNESS;
		if(f>G.MAX_BRIGHTNESS) f = G.MAX_BRIGHTNESS;
		WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.screenBrightness = f;
        getWindow().setAttributes(lp);
	}
	
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.base, menu);
		return super.onCreateOptionsMenu(menu);
	}
	
	public boolean onOptionsItemSelected(MenuItem item) {
		int id = item.getItemId();
		switch(id) {
		case R.id.menu_setting:
			openSettingDialog();
			break;
		case R.id.menu_update:
			G.toast(R.string.already_new);
			break;
		}
		return super.onOptionsItemSelected(item);
	}
	
	protected void killThread(final Thread thread) {
		new Thread(new Runnable() {
			public void run() {
				G.destroyThread(thread);
			}
		}).start();
	}
	
	protected void onStop() {
		super.onStop();
	}
	
	
}
