package rt.lanyo.game.ddz;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import rt.lanyo.game.ddz.adapter.MyAdapter;
import rt.lanyo.game.ddz.mgr.OnlineMgr;
import rt.lanyo.game.ddz.mgr.PlayerMgr;
import rt.lanyo.game.ddz.model.Card;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.service.MusicService;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.util.J;
import rt.lanyo.game.ddz.util.S;
import rt.lanyo.game.ddz.view.GameView;
import rt.lanyo.game.ddz.vo.CmdVO;
import rt.lanyo.game.ddz.vo.DeskVO;
import rt.lanyo.game.ddz.widget.DeskGrid;
import rt.lanyo.game.ddz.widget.MyDialog;
import android.app.Dialog;
import android.content.Intent;
import android.os.Message;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Gallery;
import android.widget.ImageButton;
import android.widget.ImageSwitcher;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ViewSwitcher.ViewFactory;

import com.google.gson.reflect.TypeToken;

public class Room extends Base implements OnItemSelectedListener {
	public static final byte DIALOG_INFO_SUCCESS = 2;
	public static final byte DIALOG_INFO_FAILED = 3;
	public static final byte DIALOG_CHANGE_HEAD = 4;
	
	private Player infoPlayer;
	private MyDialog exitDialog;
	private MyDialog infoDialog;
	private MyDialog joinDialog;
	private boolean login = true;
	private DeskGrid desks;
	private Type listType = new TypeToken<List<DeskVO>>(){}.getType();
	private Type joinType = new TypeToken<Map<Byte, Player>>(){}.getType();
	
	private ImageButton head;
	private RadioButton male;
	private RadioButton female;
	private EditText nick;
	private Button sure;
	private TextView nickTv;
	private MyDialog myInfoDialog;
	private MyDialog changeHeadDialog;
	private boolean modifyHead;
	private ImageSwitcher switcher;
	private int imgPosition;
	
	protected void create() {
		setContentView(R.layout.room);
		desks = (DeskGrid) findViewById(R.id.desks);
	}
	
	protected void handle(Message msg) {
		super.handle(msg);
		switch(msg.what) {
		case DIALOG_INFO_SUCCESS:
			openInfo(infoPlayer);
			break;
		case DIALOG_INFO_FAILED:
			MyDialog.change(infoDialog, R.string.tip, R.string.loading_failed, R.string.close);
			break;
		}
	}
	
	protected void deal(boolean success, CmdVO c) {
		super.deal(success, c);
		if(CmdVO.LIST.equals(c.cmd)) {
			if(success) {
				List<DeskVO> vos = J.fromJson(c.content, listType);
				desks.vos = vos;
				desks.setAdapter(Room.this);
			}
		} else if(CmdVO.JOIN.equals(c.cmd)) {
			if(!MyDialog.exist(joinDialog)) {
				OnlineMgr.sendCmd(CmdVO.LEAVE_DESK);
				return;
			}
			
			MyDialog.dismiss(joinDialog);
			Intent intent = new Intent(Room.this, Game.class);
			GameView.netPlayers = J.fromJson(c.content, joinType);
			G.netMode = true;
			startActivity(intent);
		} else if(CmdVO.LOGOUT.equals(c.cmd)) {
			OnlineMgr.close();
		} else if(CmdVO.INFO.equals(c.cmd)) {
			if(!MyDialog.exist(infoDialog)) return;
			infoPlayer = J.fromJson(c.content, Player.class);
			sendMsg(infoPlayer!=null?DIALOG_INFO_SUCCESS:DIALOG_INFO_FAILED);
		} else if(CmdVO.MODIFY.equals(c.cmd)) {
			if(CmdVO.MODIFY_SUCCESS.equals(c.content)) {
				G.toast(R.string.modify_success);
			} else {
				G.toast(R.string.modify_failed);
			}
		}
	}

	private void openInfo(Player p) {
		View view = View.inflate(this, R.layout.online_info, null);
		if(p==G.me) {
			if(p.id!=0) {
				myInfoDialog = MyDialog.create(R.string.personal, view,  R.string.sure, R.string.close);
				sure = MyDialog.getBtn(myInfoDialog, R.string.sure);
			} else {
				myInfoDialog = MyDialog.create(R.string.personal, view, R.string.close);
			}
		} else {
			infoDialog = MyDialog.change(infoDialog, R.string.personal, view,  R.string.close);
		}
		
		nick = (EditText) view.findViewById(R.id.nick);
		nick.setVisibility(View.GONE);
		nickTv = (TextView) view.findViewById(R.id.nick_tv);
		nickTv.setVisibility(View.VISIBLE);
		head = (ImageButton) view.findViewById(R.id.head);
		head.setOnClickListener(this);
		male = (RadioButton) view.findViewById(R.id.male);
		female = (RadioButton) view.findViewById(R.id.female);
		if(p!=null) { //如果已经有用户信息
			if(p.sex==null||p.sex==0) { //设置性别
				male.setChecked(true);
			} else {
				female.setChecked(true);
			}
			head.setImageResource(Info.images[S.get(p.iconIndex)]);  //设置头像
			nick.setText(p.name); //设置姓名
			
			((TextView) view.findViewById(R.id.score)).setText(p.score==null?"0":p.score.toString());
			((TextView) view.findViewById(R.id.title)).setText(p.title==null?PlayerMgr.findTitle(0):p.title);
			((TextView) view.findViewById(R.id.win_rounds)).setText(p.win==null?"0":p.win.toString());
			((TextView) view.findViewById(R.id.lose_rounds)).setText(p.lose==null?"0":p.lose.toString());
			Integer num = Integer.parseInt(
					((TextView) view.findViewById(R.id.win_rounds)).getText().toString()) 
					+ Integer.parseInt(((TextView) view.findViewById(R.id.lose_rounds)).getText().toString());
			((TextView) view.findViewById(R.id.rounds)).setText(num.toString());
			setWidgetsStatus(false);
		}
	}
	
	private void setWidgetsStatus(boolean modifying) {
		if(modifying) {  //修改
			head.setClickable(true);
			nick.setVisibility(EditText.VISIBLE);
			nickTv.setVisibility(TextView.GONE);
			nick.setText(nickTv.getText());
			if(sure!=null) sure.setText("确定");
			male.setClickable(true);
			female.setClickable(true);
		} else {
			modifyHead = false;
			head.setClickable(false);
			nick.setVisibility(EditText.GONE);
			nickTv.setVisibility(TextView.VISIBLE);
			nickTv.setText(nick.getText());
			if(sure!=null) sure.setText("修改");
			male.setClickable(false);
			female.setClickable(false);
		}
	}
	
	protected void onResume() {
		super.onResume();
		
		if(login) {
			if(G.status == G.STATUS_LOGIN) {//刚登陆进来
				Intent intent = getIntent();
				List<DeskVO> vos = J.fromJson(intent.getStringExtra(CmdVO.LIST), listType);
				desks.vos = vos;
				desks.setAdapter(Room.this);
			} else {
				OnlineMgr.sendCmd(CmdVO.LIST, null, null, this);
			}
		}
		
		G.status = G.STATUS_ROOM;
		G.startMusicService(MusicService.CMD_GO_ON);
		G.startMusicService(MusicService.TYPE_WELCOME, MusicService.CMD_PLAY);
	}

	protected void onStop() {
		super.onStop();
		if(G.status == G.STATUS_ROOM) {
			G.startMusicService(MusicService.CMD_PAUSE);
		}
	}
	
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.room, menu);
		return super.onCreateOptionsMenu(menu);
	}
	
	public boolean onOptionsItemSelected(MenuItem item) {
		super.onOptionsItemSelected(item);
		
		int id = item.getItemId();
		switch(id) { 
		case R.id.menu_personal:
			openInfo(G.me);
			break;
		}
		return true;
	}
	
	private void join(String deskId, Byte location){
		View v = LayoutInflater.from(getApplicationContext()).inflate(R.layout.progress, null);
		TextView tv = (TextView) v.findViewById(R.id.progress_tv);
		tv.setText(R.string.joining);
		joinDialog = MyDialog.create(R.string.tip, v,  R.string.cancel);
		GameView.myLocation = location;
		OnlineMgr.sendCmd(CmdVO.JOIN,deskId+":"+location,null, this);
	}
	
	
	public void logout(){
		if(login){
			OnlineMgr.sendCmd(CmdVO.LOGOUT, null, this);
			login=false;
		}
	}

	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			MusicService.playSound(Card.CLICK);
			showDialog(DIALOG_EXIT);
		}
		return super.onKeyDown(keyCode, event);
	}
	
	protected Dialog onCreateDialog(int id) {
		switch(id) {
		case DIALOG_EXIT:
			if (exitDialog == null) {
				exitDialog = MyDialog.create(R.string.tip, R.string.exit_room, R.string.yes, R.string.no);
			}
			return exitDialog.dialog;
		case DIALOG_CHANGE_HEAD:
			if(changeHeadDialog==null) {
				List<Map<Object, Object>> data = new ArrayList<Map<Object,Object>>();
				for(int i : Info.images) {
					Map<Object, Object> map = new HashMap<Object, Object>();
					map.put("id", i);
					data.add(map);
				}
				MyAdapter adapter = new MyAdapter(this, data, R.layout.head_item, new String[]{"id"}, new int[]{R.id.head}, null);
				View view = LayoutInflater.from(this).inflate(R.layout.gallery, null);
				Gallery gallery = (Gallery) view.findViewById(R.id.gallery);
		    	gallery.setAdapter(adapter); 
		    	gallery.setSelection(Info.images.length/2);
		    	gallery.setOnItemSelectedListener(this);
		    	switcher = (ImageSwitcher) view.findViewById(R.id.switcher);
		    	switcher.setFactory(new ViewFactory() {
					public View makeView() {
						ImageView view = new ImageView(Room.this);
						view.setLayoutParams(new ImageSwitcher.LayoutParams(60, 60));
						return view;
					}
				});
				changeHeadDialog = MyDialog.create(R.string.choose_head, view, R.string.sure, R.string.cancel);
			}
			return changeHeadDialog.dialog;
		}
		return super.onCreateDialog(id);
	}
	
	public void onItemSelected(AdapterView<?> parent, View view,
			int position, long id) {
		imgPosition = position;
		switcher.setImageResource(Info.images[position]);
	}
	public void onNothingSelected(AdapterView<?> arg0) {}
	
	public void onClick(View v) {
		super.onClick(v);
		if(v == MyDialog.getBtn(exitDialog, R.string.yes)) {
			logout();
			MyDialog.dismiss(exitDialog);
			finish();
		} else if (v == MyDialog.getBtn(exitDialog, R.string.no) 
				|| v==MyDialog.getBtn(infoDialog)
				|| v==MyDialog.getBtn(changeHeadDialog, R.string.close)
				|| v==MyDialog.getBtn(myInfoDialog, R.string.close)) {
			closeDialog();
		} else if(v==head) {
			showDialog(DIALOG_CHANGE_HEAD);
		} else if(v==sure) { //"确定" "修改"
			if(nick.getVisibility()==EditText.VISIBLE) {
				if(modifyHead) { //如果用户修改了头像，没有修改就用原来自己的头像
					G.me.iconIndex = imgPosition;
					modifyHead = false;
				}
				G.me.name = nick.getText().toString();
				G.me.sex = male.isChecked()?0:1;
				post(new Runnable() {
					public void run() {
						OnlineMgr.sendCmd(CmdVO.MODIFY, J.toJson(G.me), Room.this);
					}
				});
				setWidgetsStatus(false);
			} else {
				setWidgetsStatus(true);
			}
		} else if(v==MyDialog.getBtn(changeHeadDialog, R.string.sure)) { //"更改头像" -->> "确定"
			modifyHead = true;
			head.setImageResource(Info.images[imgPosition]);
			MyDialog.dismiss(changeHeadDialog);
		} else if(v==MyDialog.getBtn(changeHeadDialog, R.string.cancel)) {//"更改头像" -->> "取消"
			MyDialog.dismiss(changeHeadDialog);
		} else if(v==MyDialog.getBtn(joinDialog, R.string.cancel)) {
			MyDialog.dismiss(joinDialog);
		} else {//点了加入
			Byte location = -1;
			if(v == ((View)v.getParent().getParent()).findViewById(R.id.seat1)){
				location = Player.LEFT;
			} else if (v == ((View)v.getParent().getParent()).findViewById(R.id.seat2)){
				location = Player.RIGHT;
			} else if (v == ((View)v.getParent().getParent()).findViewById(R.id.seat3)){
				location = Player.BOTTOM;
			}
			if(location!=-1) {
				String num = ((TextView) ((View)v.getParent().getParent()).findViewById(R.id.desk_num)).getText().toString();
				DeskVO desk = null;
				for(DeskVO vo : desks.vos) {
					if(vo.id.toString().equals(num)) {
						desk = vo;
						break;
					}
				}
				if(desk!=null) {
					if(desk.left==-1&&location==Player.LEFT  
					||desk.right==-1&&location==Player.RIGHT  
					||desk.bottom==-1&&location==Player.BOTTOM ) {
						join(num, location); //加入桌子
					} else {
						info(num, location); //查看玩家信息
					}
				}
			}
		}
	}
	
	private void closeDialog() {
		MyDialog.dismiss(exitDialog);
		MyDialog.dismiss(infoDialog);
		MyDialog.dismiss(changeHeadDialog);
		MyDialog.dismiss(myInfoDialog);
	}
	
	private void info(String deskNum, int location) {
		OnlineMgr.sendCmd(CmdVO.INFO, deskNum+":"+location, null, this);
		View v = LayoutInflater.from(getApplicationContext()).inflate(R.layout.progress, null);
		TextView tv = (TextView) v.findViewById(R.id.progress_tv);
		tv.setText(R.string.rank_loading);
		infoDialog = MyDialog.create(R.string.personal, v, R.string.close);
	}
}
