package rt.lanyo.game.ddz;

import java.util.Map;

import rt.lanyo.game.ddz.adapter.MyAdapter;
import rt.lanyo.game.ddz.adapter.MyAdapter.ViewCallback;
import rt.lanyo.game.ddz.mgr.PlayerMgr;
import rt.lanyo.game.ddz.mgr.SaveMgr;
import rt.lanyo.game.ddz.mgr.TaskMgr;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.service.MusicService;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.widget.MyDialog;
import android.app.Dialog;
import android.content.Intent;
import android.util.TypedValue;
import android.view.View;
import android.widget.Button;
import android.widget.GridView;
import android.widget.TextView;

public class Tasks extends Base implements ViewCallback{
	public static final byte DIALOG_TASK_AGAIN = 2;
	
	private GridView gv;
	private Player player;
	private Integer taskId;
	private MyDialog goDialog;
	private MyDialog sureDialog;
	private MyDialog taskAgainDialog;
	
	private TextView taskStatus;
	private TextView taskRounds;
	private TextView bestRecord;
	private Button taskAgain;
	private MyAdapter adapter;
	
	protected void create() {
		setContentView(R.layout.tasks);
		
		if(taskId==null||taskId==0) taskId = 1;
		initWidgets();
	}
	
	private void initWidgets() {
		gv = (GridView) findViewById(R.id.tasks);
		taskStatus = (TextView) findViewById(R.id.task_status);
		taskRounds = (TextView) findViewById(R.id.task_rounds);
		bestRecord = (TextView) findViewById(R.id.best_record);
		taskAgain = (Button) findViewById(R.id.task_again);
	}
	
	private void initInfo() {
		player = PlayerMgr.find();
		taskId = player.taskId;
		
		int record  = new SaveMgr(this).get(SaveMgr.BEST_RECORD, 0);
		if(record!=0) {
			bestRecord.setText(String.valueOf(record));
		} else {
			bestRecord.setText("无");
		}
		if(player.taskRounds!=null && player.taskRounds>0) {
			if(taskId>TaskMgr.getTaskNums()) {
				taskStatus.setText(R.string.task_end);
			} else {
				taskStatus.setText(R.string.task_ing);
			}
			taskRounds.setText(String.valueOf(player.taskRounds));
			setBtnEnable(true);
		} else {
			taskStatus.setText(R.string.task_not_start);
			taskRounds.setText(String.valueOf(0));
			setBtnEnable(false);
		}
	}
	
	protected void onResume() {
		super.onResume();
		if(G.status!=G.STATUS_TASK) {//如果从游戏界面或者从PlayerInfo界面进来， 就要重新加载Player信息
			initInfo();
			adapter = new MyAdapter(this, R.layout.tasks_item, new int[]{R.id.btn}, this, TaskMgr.getTaskNums());
			gv.setAdapter(adapter);
		}
		G.status = G.STATUS_TASK;
		G.startMusicService(MusicService.CMD_GO_ON);
		G.startMusicService(MusicService.TYPE_WELCOME, MusicService.CMD_PLAY);
	}
	
	private void setBtnEnable(boolean able) {
		if(taskAgain==null) return;
		taskAgain.setEnabled(able);
		if(able) {
			taskAgain.setTextColor(getResources().getColorStateList(R.color.btn_text));
		} else {
			taskAgain.setTextColor(getResources().getColor(R.color.btn_d));
		}
	}
	
	protected Dialog onCreateDialog(int id) {
		switch(id) {
		case DIALOG_TASK_AGAIN:
			if(taskAgainDialog==null) {
				taskAgainDialog = MyDialog.create(R.string.tip, R.string.task_again_ask, R.string.yes, R.string.no);
			}
			return taskAgainDialog.dialog;
		}
		return super.onCreateDialog(id);
	}
	
	protected void onStop() {
		super.onStop();
		if(G.status == G.STATUS_TASK) G.startMusicService(MusicService.CMD_PAUSE);
	}

	public void onClick(View v) {
		super.onClick(v);
		if(v==MyDialog.getBtn(goDialog, R.string.begin_task)){
			G.gameMode = G.GAME_MODE_TASK;//正式进入闯关模式
			Intent intent = new Intent(Tasks.this, Game.class);
			startActivity(intent);
			MyDialog.dismiss(goDialog);
			System.gc();
		} else if(v==MyDialog.getBtn(goDialog,R.string.cancel) || v==MyDialog.getBtn(sureDialog)){ //取消按钮
			MyDialog.dismiss(goDialog);
			MyDialog.dismiss(sureDialog);
		} else if(v==taskAgain) {
			showDialog(DIALOG_TASK_AGAIN);
		} else if(v==MyDialog.getBtn(taskAgainDialog, R.string.no)) {
			MyDialog.dismiss(taskAgainDialog);
		} else if(v==MyDialog.getBtn(taskAgainDialog, R.string.yes)) {
			MyDialog.dismiss(taskAgainDialog);
			
			player.taskId = 1;
			player.taskRounds = 0;
			player.rounds = 0;
			player.loseRounds = 0;
			taskId = 1;
			adapter.notifyDataSetChanged();
			PlayerMgr.saveOrUpdate(player);
			initInfo();
		} else {
			String text = ((Button)v).getText().toString();
			if(text==null || "".equals(text)) return;  //未开启的关卡 无反应
			int id = Integer.parseInt(((Button)v).getText().toString());
			StringBuilder content = new StringBuilder(TaskMgr.taskIntros[id-1]);
			
			if(taskId>id) {
				int index = content.indexOf("\n");
				if(index != -1) content.replace(index, content.length(), "");
				sureDialog = MyDialog.create(TaskMgr.getIntroStr(id), content.append(getResources().getString(R.string.have_done)).toString(), R.string.sure);
			} else {
				goDialog = MyDialog.create(TaskMgr.getIntroStr(id), TaskMgr.getTaskContent(player), R.string.begin_task, R.string.cancel);
			}
		}
		
	}
	
	
	
	public void dealViews(Map<Integer, View> holder, int position) {
		Button btn = (Button) holder.get(R.id.btn);
		if(position+1<=taskId) { //设置为通关图片
			btn.setBackgroundResource(R.drawable.btn);
			btn.setText(String.valueOf(position+1));
			btn.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 25); //文字大小
			btn.setTypeface(G.numType); //字体
			if(position+1==taskId) {
				btn.setTextColor(getResources().getColorStateList(R.color.btn_text)); //文字颜色
			} else {
				btn.setTextColor(getResources().getColor(R.color.login_tv));
			}
		} else { //设置为禁止进入图片
			btn.setBackgroundResource(R.drawable.lock);
		}
		btn.setOnClickListener(this);
	}
	
	
}
