package rt.lanyo.game.ddz.adapter;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
/**
 * 封装出一个公共的adapter
 */
public class MyAdapter extends BaseAdapter {
	
	public Map<Integer, Map<Integer, View>> holders;
	private List<Map<Object, Object>> data;
	private int itemId;
	private Object[] from; //Map中的key
	private int[] to; //item中组建的id
	public LayoutInflater inflater;
	private ViewCallback callback;
	private int length;
	
	public MyAdapter(Context context, List<Map<Object, Object>> data,
			int itemId, Object[] from, int[] to, ViewCallback callback) {
		this.inflater = LayoutInflater.from(context);
		this.data = data;
		this.itemId = itemId;
		this.from = from;
		this.to = to;
		this.callback = callback;
		this.holders = new HashMap<Integer, Map<Integer, View>>();
		this.length = data==null?0:data.size();
	}
	
	/**
	 * 没有数据交互的Adapter
	 * @param context
	 * @param itemId
	 * @param to
	 * @param callback
	 */
	public MyAdapter(Context context, int itemId, int[] to, ViewCallback callback, int length) {
		this.inflater = LayoutInflater.from(context);
		this.itemId = itemId;
		this.to = to;
		this.callback = callback;
		this.holders = new HashMap<Integer, Map<Integer, View>>();
		this.length = length;
	}
	
	public MyAdapter() {}
	
	public int getCount() {
		return length;
	}

	public Object getItem(int position) {
		if(holders==null || holders.isEmpty()) return null;
		return holders.get(position);
	}

	public long getItemId(int position) {
		return position;
	}

	public View getView(int position, View convertView, ViewGroup parent) {
		convertView = inflater.inflate(itemId, null);
		Map<Integer, View> holder = new HashMap<Integer, View>();
		for(int id : to) {
			holder.put(id, convertView.findViewById(id));
		}
		holders.put(position, holder);
		if(data!=null && !data.isEmpty()) {
			Map<Object, Object> map = data.get(position); //获得第position组的数据
			if(map!=null) {
				for(int i=0; i<to.length; i++) { //遍历每个组件
					if(i+1>from.length) continue; //如果第i个组件没有对应的内容
					Object o = map.get(from[i]); //获得第i个组件的内容
					if(o==null) continue; //如果内容为空
					View v = holder.get(to[i]); //获得第i个组件
					if(v instanceof TextView) {
						((TextView) v).setText(o.toString());
					} else if(v instanceof ImageButton) {
						((ImageButton)v).setImageResource(Integer.valueOf(o.toString()));
					} else if(v instanceof ImageView) {
						((ImageView)v).setImageResource(Integer.valueOf(o.toString()));
					} else if(v instanceof Button) {
						((Button)v).setBackgroundResource(Integer.valueOf(o.toString()));
					} else if(v instanceof CheckBox) {
						((CheckBox)v).setChecked(Boolean.valueOf(o.toString()));
					}
				}
			}
		}
		
		if(callback!=null && holder!=null) callback.dealViews(holder, position);
		return convertView;
	}
	
	public interface ViewCallback {
		void dealViews(Map<Integer, View> holder, int position);
	}
}
