package rt.lanyo.game.ddz.mgr;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import rt.lanyo.game.ddz.R;
import rt.lanyo.game.ddz.model.Msg;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.util.G;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.RectF;
import android.os.Handler;
/**
 * 管理信息
 */
public class MsgMgr {
	public static List<Msg> messages = Collections.synchronizedList(new ArrayList<Msg>());
	public static Map<Integer, Msg> readyMsgs;
	private static String readyStr = G.context.getString(R.string.ready);
	public static Handler handler;
	
	public static void append(Player player,Object msg,long delay){
		Msg m = null;
		Object param = getParam(msg);
		int width = param.toString().length() * G.msgSize + G.MSG_MARGIN*2;
		int height = G.msgSize*3;
		int x = player.msgX;
		if(G.me.next==player) {
			x = player.x - width - G.MSG_MARGIN;
		}
		m = new Msg(
				x,  player.msgY,
				width, height,  
				param.toString());
		messages.add(m);
		
		
		
		if(delay>=0) {
			final Msg temp = m;
			handler.postDelayed(new Runnable() {
				public void run() {
					for (Iterator<Msg> iterator = messages.iterator(); iterator.hasNext();) {
						if(temp==iterator.next()) iterator.remove();//要一条一条的删
					}
				}
			},delay>0?delay:G.MSG_TIME);
		}
		
	}
	
	public static void append(Player player,Object msg){
		append(player, msg, 0);
	}
	
	public static void drawMsgs(Canvas canvas, Paint pt) {
		float size = pt.getTextSize();
		pt.setColor(G.msgColor);
		pt.setTextSize(G.msgSize);
		synchronized (messages) {
			for(Msg msg:messages){
				drawMsg(msg, canvas, pt);
			}
		}
		pt.setTextSize(size);
	}
	
	private static void drawMsg(Msg msg, Canvas canvas, Paint pt) {
		int y = msg.y;
		int x = msg.x;
		if(msg.y>G.winHeight/2) {
			y += ImgMgr.clock.getHeight();  //如果是我,信息放下一点  防止遮到闹钟
			x += G.MSG_MARGIN*3;//如果是我,信息放右一点  防止遮到闹钟
		}
		int color = pt.getColor();
		pt.setColor(G.context.getResources().getColor(R.color.login_solid));
		canvas.drawRoundRect(new RectF(x, y, x+msg.w, y+msg.h), 5, 5, pt);
		Path path = new Path();
		if(x>G.winWidth/2) {//右边的
			path.moveTo(x+msg.w+10, y+3);
			path.lineTo(x+msg.w, y+msg.h/3);
			path.lineTo(x+msg.w, y+msg.h*2/3);
		} else if(y<G.winHeight/2) {
			path.moveTo(x-10, y+3);
			path.lineTo(x, y+msg.h/3);
			path.lineTo(x, y+msg.h*2/3);
		} else {//Me
			path.moveTo(x-10, y+msg.h-3);
			path.lineTo(x, y+msg.h/3);
			path.lineTo(x, y+msg.h*2/3);
		}
		path.close();
		canvas.drawPath(path, pt);
		pt.setColor(color);
		if(msg.msg!=null) {
			int x2 = msg.x + G.MSG_MARGIN;
			int y2 = msg.y+msg.h*7/12;
			if(msg.y>G.winHeight/2) {
				y2 +=ImgMgr.clock.getHeight();  //如果是我,信息放下一点  防止遮到闹钟
				x2 += G.MSG_MARGIN*3;//如果是我,信息放右一点  防止遮到闹钟
			}
			canvas.drawText(msg.msg, x2, y2, pt);
		}
	}
	
	/**
	 * 解析参数
	 */
	private static Object getParam(Object obj) {
		if(obj instanceof String) {
			return obj.toString();
		} else if(obj instanceof Integer){
			return G.context.getResources().getString(Integer.valueOf(obj.toString()));
		} else {
			return obj==null?"":obj;
		}
	}
	
	public static void drawReady(List<Player> players, Canvas canvas, Paint pt) {
		if(readyMsgs==null) return;
		synchronized (players) {
			for(Player player : players) {
				if(readyMsgs.get(player.id)==null) {
					int width = readyStr.length() * G.msgSize + G.MSG_MARGIN*2;
					int height = G.msgSize*3;
					int x = player.msgX;
					if(G.me.next==player) {
						x = player.x - width - G.MSG_MARGIN;
					}
					Msg m = new Msg(
							x,  player.msgY,
							width, height,  
							readyStr);
					readyMsgs.put(player.id, m);
				}
				
				if(player.ready) {
					player.cleanCard();
					drawMsg(readyMsgs.get(player.id), canvas, pt);
				}
			}
		}
	}
	
	public static void clearReadys(List<Player> players) {
		if(players==null) return;
		synchronized (players) {
			for(Player player : players) {
				player.ready = false;
			}
			synchronized (readyMsgs) {
				if(readyMsgs!=null) readyMsgs.clear();
			}
		}
	}
}
