package rt.lanyo.game.ddz.mgr;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import rt.lanyo.game.ddz.R;
import rt.lanyo.game.ddz.model.Player;
import rt.lanyo.game.ddz.util.G;
import rt.lanyo.game.ddz.util.J;

import com.google.gson.reflect.TypeToken;
/**
 * 管理Player信息:单机、帐号管理
 * @author MJ
 */
public class PlayerMgr implements Comparator<Player>{
	public static String accounts_setting;
	public static String single_setting;
	public static final byte FIRST_ID = 1;
	
	private static Type listType;
	private static String[] titles;
	private static int[] scores;
	private static SaveMgr saveMgr;
	private static PlayerMgr mgr;
	
	static {
		if(scores==null) scores = G.context.getResources().getIntArray(R.array.score);
		if(titles==null) titles = G.context.getResources().getStringArray(R.array.title);
		if(accounts_setting==null) accounts_setting = G.context.getString(R.string.accounts_setting);
		if(single_setting==null) single_setting = G.context.getString(R.string.single_setting);
		if(listType==null) listType = new TypeToken<List<Player>>(){}.getType();
		if(mgr==null) mgr = new PlayerMgr();
	}
	
	public static int saveOrUpdate(Player player) {
		if(player==null) return -1;
		String s = null;
		int ret = -1;
		Player old = find();
		if(player.id==null || player.id<FIRST_ID) {//添加用户
			if(old==null || old.id==null || old.id<FIRST_ID) {//安装游戏以来第一次创建用户
				player.id = (int)FIRST_ID;
			} else {
				player.id = old.id+1;
			}
			player.score = Player.INIT_SCORE;
			player.title = findTitle(Player.INIT_SCORE);
			player.taskId = 1;
			player.rounds = 0;
			player.taskRounds = 0;
			player.loseRounds = 0;
			ret = player.id;
			s = J.toJson(player);
		} else { //更新用户
			if(old==null || old.id==null || old.id<FIRST_ID) {//游戏数据无故消失
				old = new Player();
				old.id = (int)FIRST_ID;
			}
			if(player.name!=null) old.name = player.name;
			if(player.title!=null) old.title = player.title;
			if(player.score!=null) {
				old.score = player.score;
				old.title = findTitle(player.score); //根据积分设置头衔
			}
			if(player.iconIndex!=null) old.iconIndex = player.iconIndex;
			if(player.sex!=null) old.sex = player.sex;
			
			//以下是跟闯关有关的属性
			if(player.taskId!=null) old.taskId = player.taskId;
			if(player.rounds!=null) old.rounds = player.rounds;
			if(player.taskRounds!=null) old.taskRounds = player.taskRounds;
			if(player.loseRounds!=null) old.loseRounds = player.loseRounds;
			if(player.leftScore!=null) old.leftScore = player.leftScore;
			if(player.rightScore!=null) old.rightScore = player.rightScore;
			ret = old.id;
			s = J.toJson(old);
		}
		save(single_setting, s);
		return ret;
	}
	
	public static boolean updateScore(Player player) {
		Player p = find();
		String lastTitle = p.title;
		Integer lastScore = p.score==null? 0: p.score;
		
		Integer score = player.score==null?0:player.score;
		String title = findTitle(score);
		player.title = title; //方便屏幕上头衔的刷新
		
		p.score = score;
		p.title = title;
		save(single_setting, J.toJson(p));
		return (!title.equals(lastTitle)) && score>lastScore;  //升级了
	}
	
	public static Player find() {
		String s = new SaveMgr(G.context).get(single_setting, null);
		if(s==null || "".equals(s.trim())) return null;
		return J.fromJson(s, Player.class);
	}
	
	public static void delete() {
		save(single_setting, null);
	}
	
	/**
	 * 根据分数来查找等级
	 * @param score
	 * @return
	 */
	public static String findTitle(Integer score) {
		score = score==null?0:score;
		if(score>scores[scores.length-1]) return titles[titles.length-1];
		int index = 0;
		for(int i=0; i<scores.length; i++) {
			if(scores[i]>score) {
				index = i-1;
				break;
			}
		}
		index = index<0?0:index;
		return titles[index];
	}
	
	/**获得以前登陆成功过的帐号**/
	public static List<Player> getLoginPlayers() {
		String s = new SaveMgr(G.context).get(accounts_setting, null);
		if(s==null || "".equals(s.trim())) return null;
		List<Player> list = J.fromJson(s, listType);
		if(list == null || list.isEmpty()) return null;
		Collections.sort(list, mgr);
		return list;
	}
	
	/**保存登陆成功的帐号**/
	public static void saveLoginPlayer(Player vo) {
		if(vo==null || vo.account==null || vo.pwd==null) return;
		List<Player> list = getLoginPlayers();
		if(list==null) list = new ArrayList<Player>();
		
		for (Iterator<Player> iterator = list.iterator(); iterator.hasNext();) {
			Player v = (Player) iterator.next();
			if(v.account.equals(vo.account)) {//如果保存过
				iterator.remove(); //移除以前的
			}
		}
		list.add(vo);
		save(accounts_setting, J.toJson(list));
	}
	
	/**删除保存的帐号**/
	public static void delLoginPlayer(String account) {
		List<Player> list = getLoginPlayers();
		if(list==null) return;
		
		for(Iterator<Player> it = list.iterator();it.hasNext();) {
			Player v = it.next();
			if(v.account.equals(account)) {
				it.remove();
			}
		}
		save(accounts_setting, J.toJson(list));
	}
	
	/**删除保存的帐号**/
	public static void delLoginPlayer(List<String> accounts) {
		for (Iterator<String> iterator = accounts.iterator(); iterator.hasNext();) {
			String account = iterator.next();
			delLoginPlayer(account);
		}
	}
	
	/**获得最后一次登陆的帐号,下次启动应用时显示**/
	public static Player getLastLoginPlayer() {
		List<Player> list = getLoginPlayers();
		if(list==null) return null;
		return list.get(0);
	}

	public int compare(Player object1, Player object2) {
		if(object1!=null && object1.lastLoginTime!=null && object2!=null && object2.lastLoginTime!=null) {
			return (int)(object2.lastLoginTime-object1.lastLoginTime);
		}
		return 0;
	}
	
	private static void save(String file, String value) {
		saveMgr = new SaveMgr(G.context);
		saveMgr.update(file, value);
		saveMgr.commit();
	}
	
	/**注册验证**/
	public static Player validateRegister(String account, String pwd, String pwd_confirm, int imgPosition, int sex){
		if(account == null || "".equals(account)) {
			G.toast(R.string.no_account);
			return null;
		}
		if(account.indexOf(" ") != -1) {
			G.toast(R.string.account_error);
			return null;
		}
		if(account.length() < 6) {
			G.toast(R.string.account_short);
			return null;
		}
		if(pwd == null || "".equals(pwd)) {
			G.toast(R.string.no_pwd);
			return null;
		}
		if(pwd.indexOf(" ") != -1) {
			G.toast(R.string.pwd_error);
			return null;
		}
		if(pwd.length() < 6) {
			G.toast(R.string.pwd_short);
			return null;
		}
		if(pwd_confirm == null || "".equals(pwd_confirm)) {
			G.toast(R.string.no_confirm_pwd);
			return null;
		}
		if(!pwd.equals(pwd_confirm)) {
			G.toast(R.string.pwd_different);
			return null;
		}
		return new Player(account, account, pwd_confirm, imgPosition, sex);
	}
	
	public static Player validateLogin(String account, String pwd) {
		if(account == null || "".equals(account)) {
			G.toast(R.string.no_account);
			return null;
		} 
		if(account.indexOf(" ") != -1) {
			G.toast(R.string.account_error);
			return null;
		}
		if(account.length() < 6) {
			G.toast(R.string.account_short);
			return null;
		}
		if(pwd == null || "".equals(pwd)) {
			G.toast(R.string.no_pwd);
			return null;
		}
		if(pwd.indexOf(" ") != -1) {
			G.toast(R.string.pwd_error);
			return null;
		}
		if(pwd.length() < 6) {
			G.toast(R.string.pwd_short);
			return null;
		}
		
		return new Player(account, pwd);
	}
	
	public static Player findPlayer(List<Player> players, float x, float y) {
		synchronized (players) {
			for(Player p : players) {
				if(x>p.x&&x<(p.x+p.icon.getWidth())  
				&& y>p.y&&y<(p.y+p.icon.getHeight())) {
					return p;
				}
			}
		}
		return null;
	}
}
